@@ -159,6 +159,10 @@ class FleurXMLModifier:
             'xml_set_text_no_create': self.xml_set_text_no_create,
             'set_nmmpmat': self.set_nmmpmat,
             'rotate_nmmpmat': self.rotate_nmmpmat,
+            'set_nkpts': self.set_nkpts,
+            'set_kpath': self.set_kpath,
+            'set_kpointlist': self.set_kpointlist,
+            'switch_kpointset': self.switch_kpointset,
         }
         return outside_actions
 
@@ -355,7 +359,7 @@ class FleurXMLModifier:
         Appends a :py:func:`~masci_tools.util.xml.xml_setters_names.create_tag()` to
         the list of tasks that will be done on the xmltree.
 
-        :param tag_name: str of the tag to create
+        :param tag: str of the tag to create or etree Element with the same name
         :param complex_xpath: an optional xpath to use instead of the simple xpath for the evaluation
         :param create_parents: bool optional (default False), if True and the given xpath has no results the
                                the parent tags are created recursively
@@ -619,3 +623,64 @@ class FleurXMLModifier:
         :param theta: float, angle (radian), by which to rotate the density matrix
         """
         self._tasks.append(ModifierTask('rotate_nmmpmat', args, kwargs))
+
+    def set_kpointlist(self, *args, **kwargs):
+        """
+        Appends a :py:func:`~masci_tools.util.xml.xml_setters_names.set_kpointlist()` to
+        the list of tasks that will be done on the xmltree.
+
+        .. warning::
+            For input versions Max4 and older **all** keyword arguments are not valid (`name`, `kpoint_type`,
+            `special_labels`, `switch` and `overwrite`)
+
+        :param kpoints: list or array containing the **relative** coordinates of the kpoints
+        :param weights: list or array containing the weights of the kpoints
+        :param name: str for the name of the list, if not given a default name is generated
+        :param kpoint_type: str specifying the type of the kPointList ('path', 'mesh', 'spex', 'tria', ...)
+        :param special_labels: dict mapping indices to labels. The labels will be inserted for the kpoints
+                               corresponding to the given index
+        :param switch: bool, if True the kPointlist will be used by Fleur when starting the next calculation
+        :param overwrite: bool, if True and a kPointlist with the given name already exists it will be overwritten
+        """
+        self._tasks.append(ModifierTask('set_kpointlist', args, kwargs))
+
+    def switch_kpointset(self, *args, **kwargs):
+        """
+        Appends a :py:func:`~masci_tools.util.xml.xml_setters_names.switch_kpointset()` to
+        the list of tasks that will be done on the xmltree.
+
+        .. warning::
+            This method is only supported for input versions after the Max5 release
+
+        :param list_name: name of the kPoint set to use
+        """
+        self._tasks.append(ModifierTask('switch_kpointset', args, kwargs))
+
+    def set_nkpts(self, *args, **kwargs):
+        """
+        Appends a :py:func:`~masci_tools.util.xml.xml_setters_names.set_nkpts()` to
+        the list of tasks that will be done on the xmltree.
+
+        .. warning::
+            This method is only supported for input versions before the Max5 release
+
+        :param count: number of k-points
+        :param gamma: bool that controls if the gamma-point should be included
+                      in the k-point mesh
+        """
+        self._tasks.append(ModifierTask('set_nkpts', args, kwargs))
+
+    def set_kpath(self, *args, **kwargs):
+        """
+        Appends a :py:func:`~masci_tools.util.xml.xml_setters_names.set_kpath()` to
+        the list of tasks that will be done on the xmltree.
+
+        .. warning::
+            This method is only supported for input versions before the Max5 release
+
+        :param kpath: a dictionary with kpoint name as key and k point coordinate as value
+        :param count: number of k-points
+        :param gamma: bool that controls if the gamma-point should be included
+                      in the k-point mesh
+        """
+        self._tasks.append(ModifierTask('set_kpath', args, kwargs))
